%{
#include <stdlib.h>
#include <string.h>
#include "mikroc.h"
%}

%union {
  char val[STRLEN];
  int num;
}

%token <val> IDENT NUMBER 
%token <num> EQ NE LE GE ADD SUB MUL DIV
%token <num> PRINT PRINTX PRINTC WHILE IF ELSE DO
%token <num> INV
%type <val> expr assign
%type <val> or or1 and and1 eq eq1 less less1 add add1 mul mul1 
%type <val> term

%%

program:
'{' { gen_out_start(); } stmtlist { gen_out_end(); } '}'
;

stmtlist:
stmt stmtlist
|
;

stmt:
PRINT expr ';' { gen_op(PRINT, NULL, $2, NULL); }
| PRINTX expr ';' { gen_op(PRINTX, NULL, $2, NULL); }
| PRINTC expr ';' { gen_op(PRINTC, NULL, $2, NULL); }
| expr ';' {}
| WHILE '(' { $<num>$ = new_label(); gen_label($<num>$); } expr ')' 
{ $<num>$ = new_label(); gen_jmp($4, 0, $<num>$); } stmt 
{ gen_jmp(NULL, 0, $<num>3); gen_label($<num>6); }
| DO { $<num>$ = new_label(); gen_label($<num>$); } stmt WHILE expr 
{ gen_jmp($5, 1, $<num>2); } ';'
| IF '(' expr ')' { $<num>$ = new_label(); gen_jmp($3, 0, $<num>$); } stmt 
{ $<num>$ = new_label(); gen_jmp(NULL, 0, $<num>$); gen_label($<num>5); } 
if_else
| '{' stmtlist '}'
| ';'
;

if_else:
{ gen_label($<num>0); }
| ELSE stmt { gen_label($<num>0); }
;

expr: 
assign
| '-' '(' expr ')' { new_var($$); gen_op(INV, $$, $3, NULL); }
;

assign:
IDENT '=' assign { 
  snprintf($$, STRLEN, "[%s]", $1); gen_op('=', $$, $3, NULL);
}
| IDENT ADD assign {
  snprintf($$, STRLEN, "[%s]", $1); gen_op(ADD, $$, $3, NULL);
}
| IDENT SUB assign {
  snprintf($$, STRLEN, "[%s]", $1); gen_op(SUB, $$, $3, NULL);
}
| IDENT MUL assign {
  snprintf($$, STRLEN, "[%s]", $1); gen_op(MUL, $$, $3, NULL);
}
| IDENT DIV assign {
  snprintf($$, STRLEN, "[%s]", $1); gen_op(DIV, $$, $3, NULL);
}
| or { strncpy($$, $1, STRLEN); }
;

or:
and or1 { cond_copy($$, $2, $1); }
;

or1:
'|' and { new_var($$); gen_op('|', $$, $<val>0, $2); } or1
{ cond_copy($$, $4, $<val>3); }
| { *$$ = 0; }
;

and:
eq and1 { cond_copy($$, $2, $1); }
;

and1:
'&' eq { new_var($$); gen_op('&', $$, $<val>0, $2); } and1
{ cond_copy($$, $4, $<val>3); }
| { *$$ = 0; }
;

eq:
less eq1 { cond_copy($$, $2, $1); }
;

eq1:
EQ less { new_var($$); gen_op(EQ, $$, $<val>0, $2); } eq1
{ cond_copy($$, $4, $<val>3); }
| NE less { new_var($$); gen_op(NE, $$, $<val>0, $2); } eq1
{ cond_copy($$, $4, $<val>3); }
| { *$$ = 0; }
;

less:
add less1 { cond_copy($$, $2, $1); }
;

less1:
'<' eq { new_var($$); gen_op('<', $$, $<val>0, $2); } less1
{ cond_copy($$, $4, $<val>3); }
| '>' eq { new_var($$); gen_op('>', $$, $<val>0, $2); } less1
{ cond_copy($$, $4, $<val>3); }
| LE eq { new_var($$); gen_op(LE, $$, $<val>0, $2); } less1
{ cond_copy($$, $4, $<val>3); }
| GE eq { new_var($$); gen_op(GE, $$, $<val>0, $2); } less1
{ cond_copy($$, $4, $<val>3); }
| { *$$ = 0; }
;

add:
mul add1 { cond_copy($$, $2, $1); }
;

add1:
'+' mul { new_var($$); gen_op('+', $$, $<val>0, $2); } add1
{ cond_copy($$, $4, $<val>3); }
| '-' mul { new_var($$); gen_op('-', $$, $<val>0, $2); } add1
{ cond_copy($$, $4, $<val>3); }
| { *$$ = 0; }
;

mul:
term mul1 { cond_copy($$, $2, $1); }
;

mul1:
'*' term { new_var($$); gen_op('*', $$, $<val>0, $2); } mul1 
{ cond_copy($$, $4, $<val>3); }
| '/' term { new_var($$); gen_op('/', $$, $<val>0, $2); } mul1 
{ cond_copy($$, $4, $<val>3); }
| { *$$ = 0; }
;

term:
'(' expr ')' { strncpy($$, $2, STRLEN); }
| IDENT { snprintf($$, STRLEN, "[%s]", $1); }
| NUMBER { strncpy($$, $1, STRLEN); }
;
