%option noyywrap
%option yylineno

%{
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "mikroc.h"
#include "y.tab.h"
%}

blank [ \t]
ident [a-zA-Z]
digit [0-9]
number {digit}+
xdigit [0-9a-fA-F]
xnumber 0(x|X){xdigit}+

%%

{blank}

{ident} { strncpy(yylval.val, yytext, 10); return IDENT; } 

{number} { strncpy(yylval.val, yytext, 10); return NUMBER; }

{xnumber} { 
  int d; sscanf(yytext, "%x", &d); snprintf(yylval.val, 10, "%i", d); 
  return NUMBER;
}

\'[^\']\' { snprintf(yylval.val, 10, "%i", yytext[1]); return NUMBER; }

"*" { return '*'; }

"/" { return '/'; }

"+" { return '+'; }

"-" { return '-'; }

"==" { return EQ; }

"!=" { return NE; }

"<" { return '<'; }

">" { return '>'; }

"<=" { return LE; }

">=" { return GE; }

"&" { return '&'; }

"|" { return '|'; }

"=" { return '='; }

"+=" { return ADD; }

"-=" { return SUB; }

"*=" { return MUL; }

"/=" { return DIV; }

";" { return ';'; }

"(" { return '('; }

")" { return ')'; }

"{" { return '{'; }

"}" { return '}'; }

print { return PRINT; }

printx { return PRINTX; }

printc { return PRINTC; }

while { return WHILE; }

if { return IF; }

else { return ELSE; }

do { return DO; }

"/*" { 
  char c, c_prev = 0;
  while (c = 
#ifdef __cplusplus
	 yyinput()
#else
	 input()
#endif
	 ) { 
    if (c == EOF) { printf("\nEOF in comment\n"); exit(1); }
    if ((c_prev == '*') && (c == '/')) break;
    c_prev = c;
    if (c == '\n') yylineno++;
  }
}

. { 
  printf("\nUnknown character at line %i: ", yylineno);
  if (*yytext > ' ') printf("'%c'\n", *yytext);
  else printf("0x%X\n", *yytext);
  exit(1);
}
